"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const radash_1 = __importDefault(require("radash"));
const pulumi = __importStar(require("@pulumi/pulumi"));
const aws = __importStar(require("@pulumi/aws"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const cmdish_1 = __importDefault(require("cmdish"));
const pulumi_aws_code_build_1 = require("@exobase/pulumi-aws-code-build");
const pulumi_aws_lambda_api_1 = require("@exobase/pulumi-aws-lambda-api");
const octokit_downloader_1 = __importDefault(require("octokit-downloader"));
const main = () => __awaiter(void 0, void 0, void 0, function* () {
    //
    //  READ PROJECT CONFIG
    //
    const context = yield fs_extra_1.default.readJSON('./context.json');
    const { platform, service, deployment } = context;
    const config = deployment.config.stack;
    //
    //  SETUP PROVIDER
    //
    const provider = new aws.Provider('aws', {
        secretKey: platform.providers.aws.accessKeySecret,
        accessKey: platform.providers.aws.accessKeyId,
        region: platform.providers.aws.region
    });
    //
    //  CREATE SOURCE ZIP
    //
    yield installSourceDependencies();
    //
    //  CREATE CODE BUILD PROJECT
    //
    const project = new pulumi_aws_code_build_1.AWSCodeBuildProject(radash_1.default.dashCase(service.name), {
        sourceDir: `${__dirname}/source`,
        buildTimeoutSeconds: toNumber(config.buildTimeoutSeconds),
        buildCommand: config.buildCommand,
        image: config.dockerImage,
        environmentVariables: deployment.config.environmentVariables.map(ev => ({
            name: ev.name,
            value: ev.value
        }))
    }, { provider });
    if (!config.useBridgeApi) {
        return {
            url: null
        };
    }
    //
    //  CREATE BRIDGE API
    //
    //  A todo here is to version the bridge. Pull from specific
    //  release version branch.
    //
    yield octokit_downloader_1.default.download({
        from: 'https://github.com/exobase-inc/aws-cloud-build-trigger-bridge',
        to: `${__dirname}/bridge.zip`,
        unzip: true
    });
    const api = new pulumi_aws_lambda_api_1.AWSLambdaAPI(radash_1.default.dashCase(`${service.name}-bridge`), {
        sourceDir: `${__dirname}/bridge`,
        sourceExt: 'ts',
        timeout: 1,
        memory: 128,
        runtime: 'nodejs14.x',
        distDirName: 'build',
        buildCommand: (() => {
            const useNvm = !!process.env.USE_NVM;
            const nvmPrefix = 'source ~/.nvm/nvm.sh && nvm use && ';
            const cmd = 'yarn && yarn build && cp package.json ./build/package.json && cd build && yarn';
            return `${useNvm ? nvmPrefix : ''}${cmd}`;
        })(),
        environmentVariables: {
            AWS_CODE_BUILD_PROJECT_NAME: project.project.name,
            BRIDGE_API_KEY: config.bridgeApiKey
        },
        domain: service.domain
    }, { provider });
    const lambdaStartBuildPolicy = new aws.iam.Policy("lambdaStartBuild", {
        path: "/",
        description: "IAM policy for logging from a lambda",
        policy: pulumi.interpolate `{
    "Version": "2012-10-17",
    "Statement": [
      {
        "Action": [
          "codebuild:StartBuild"
        ],
        "Effect": "Allow",
        "Resource": "${project.project.arn}"
      }
    ]
  }
  `
    }, { provider });
    new aws.iam.RolePolicyAttachment("lambdaStartBuilds", {
        role: api.role.name,
        policyArn: lambdaStartBuildPolicy.arn,
    }, { provider });
    return {
        url: service.domain ? service.domain.fqd : api.api.url
    };
});
const installSourceDependencies = () => __awaiter(void 0, void 0, void 0, function* () {
    const USE_NVM = !!process.env.USE_NVM;
    //
    // Install dependencies
    //
    if (USE_NVM) {
        const [err] = yield (0, cmdish_1.default)('source ~/.nvm/nvm.sh && nvm use && yarn', {
            cwd: `${__dirname}/source`
        });
        if (err)
            throw err;
    }
    else {
        const [err] = yield (0, cmdish_1.default)('yarn', {
            cwd: `${__dirname}/source`
        });
        if (err)
            throw err;
    }
});
const toNumber = (value) => {
    if (radash_1.default.isString)
        return parseInt(value);
    return value;
};
exports.default = main;
//# sourceMappingURL=index.js.map